@@ -1,5 +1,7 @@
 # coding: utf-8
 
+import gzip
+
 from vxci.get_addons import get_module_list, get_modules2test, get_tags, join_prefix_suffix
 from vxbase import vtypes, net, fromenv
 from vxci.tools_test import install_chrome_commands
@@ -17,6 +19,7 @@ import time
 from urllib3.exceptions import ReadTimeoutError
 from unicodedata import normalize
 from shutil import copy, rmtree
+from jinja2 import Template
 
 
 _logger = logging.getLogger('vxci.' + __name__)
@@ -26,6 +29,7 @@ TO_SCAN = ['git.vauxoo.com',
            'gitlab.com',
            'bitbucket.org',
            'git.islamicreliefcanada.ca']
+INSTANCE_TYPES = ['develop', 'test', 'updates']
 
 
 def pull_images(images):
@@ -396,6 +400,19 @@ def get_coverage_env_variables(config):
     return coverage_info
 
 
+def odoo_config2psql_env(config):
+    """Translate odoo configuration parameters to psql environment variables
+    In order to be able to run 'psql' command directly without extra parameters
+    to connect to databases"""
+    psql_env = {}
+    for key, value in config.items():
+        if not key.startswith("DB_"):
+            continue
+        new_key = key.replace("DB_", "PG") if key != "DB_NAME" else "PG_DATABASE"
+        psql_env[new_key] = value
+    return psql_env
+
+
 def start_instance(config):
     coverage_info = get_coverage_env_variables(config)
     env = {
@@ -407,6 +424,8 @@ def start_instance(config):
         # "COVERAGE_DEBUG": "config,dataio,process",
     }
     env.update(coverage_info)
+    psql_env = odoo_config2psql_env(env)
+    env.update(psql_env)
     config.update({key.lower(): value for key, value in coverage_info.items()})
     for env_var in ['COUNTRY', 'LANGUAGE']:
         env.update({env_var: environ.get(env_var, "")})
@@ -547,11 +566,13 @@ def container_coverage_reports(config, combine=False):
         cmd = '/bin/sh -c "coverage combine --keep {coverage_reports}/.coverage_*"'.format(**config)
         cover_reports.append(cmd)
 
-    cover_reports.extend([
-        'coverage html -d {coverage_reports}/htmlcov'.format(**config),
-        'coverage xml -o {coverage_reports}/coverage.xml'.format(**config),
-        'coverage report -m --show-missing',
-    ])
+    cover_reports.append('coverage report -m --show-missing')
+    if combine:
+        # Only get html and xml in push_coverage job
+        cover_reports.extend([
+            'coverage html -d {coverage_reports}/htmlcov'.format(**config),
+            'coverage xml -o {coverage_reports}/coverage.xml'.format(**config),
+        ])
 
     status, msg = (True, "")
     for cr in cover_reports:
@@ -594,12 +615,25 @@ def test_module(config):
     test_res = exec_cmd(config['container_name'], test_command, 'odoo', stream=True, workdir=config.get('coverage_home'))
 
     test_log = resume_log(test_res, config['allow_deprecated'])
+    success = test_log[0]
     show_log(test_log[1], 'Installation with demo and tests', config)
     save_logs(config, test_res)
 
     container_coverage_reports(config)
 
-    return test_log[0]
+    # Parse template for all instance types and run queries for each one
+    for instance_type in INSTANCE_TYPES:
+        _logger.info("Deactivating database using instance type: %s", instance_type)
+        queries = deactivate2queries(instance_type)
+        for query in queries:
+            cmd_query = ["/bin/sh", "-c", 'psql test -XtAc "%s" && echo "pass"' % query]
+            query_res = exec_cmd(config['container_name'], cmd_query, 'odoo')
+            query_res_lines = query_res.splitlines()
+            if "pass" in query_res_lines[-1:]:
+                continue
+            _logger.error('query error "%s" Result: "%s"', query, query_res_lines[:-1])
+            success = False
+    return success
 
 def copy_files(src, dst):
     for file_name in listdir(src):
@@ -641,11 +675,11 @@ def prepare_coverage(config):
 
 
 def save_logs(config, logs):
-    odoo_log = path.join(config.get('container_vol'), 'odoo.log_%(ci_job_name)s' % config)
+    odoo_log = path.join(config.get('container_vol'), 'odoo.log.gz_%(ci_job_name)s' % config)
     _logger.info("Save log in %s cwd %s", odoo_log, getcwd())
-    with open(odoo_log, "a") as fodoo_log:
+    with gzip.open(odoo_log, "wb") as fodoo_log:
         for line in logs:
-            fodoo_log.write(line)
+            fodoo_log.write(line.encode("UTF-8"))
 
 
 def get_coverage_content(config):
@@ -722,6 +756,8 @@ def resume_log(log_lines, allow_deprecated=False):
     color_regex = re.compile(r'\x1B\[([0-9]{1,2}(;[0-9]{1,2})?)?[m|K]')
     is_magic_line = False
     for line in log_lines:
+        if isinstance(line, bytes):
+            line = line.decode("UTF-8")
         original_line = line.strip()
         if 'odoo.modules.loading: Modules loaded.' in original_line:
             is_magic_line = True
@@ -768,3 +804,17 @@ def generate_coverage_reports(config):
     status, msg = container_coverage_reports(config, combine=True)
     clean_containers(config)
     return status, msg
+
+
+def deactivate2queries(instance_type):
+    deactivate_path = 'deactivate.jinja'
+    if not path.isfile(deactivate_path):
+        _logger.debug("Deactivate file %s does not exists", deactivate_path)
+        return []
+    with open(deactivate_path) as f_deactivate:
+        template_str = f_deactivate.read()
+    template = Template(template_str)
+    rendered_template = template.render(instance_type=instance_type)
+    queries = [q[1] for q in sorted(json.loads(rendered_template).items())]
+    _logger.debug("Template rendered for instance type %s:\n%s\n", instance_type, '\n'.join(queries))
+    return queries
